/***************************************************************************
 *   Copyright (C) 2006 by Stefan Hirche                                   *
 *   hirche@astro.univie.ac.at                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "logik.h"
#include "pref.h"
#include "myview.h"

#include <qdragobject.h>
#include <kprinter.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qcanvas.h>

#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kdeversion.h>
#include <kstatusbar.h>
#include <kaccel.h>
#include <kio/netaccess.h>
#include <kfiledialog.h>
#include <kconfig.h>
#include <kurl.h>
#include <kurldrag.h>
#include <kurlrequesterdlg.h>
#include <kmessagebox.h>

#include <kstdaccel.h>
#include <kaction.h>
#include <kstdaction.h>

logik::logik()
    : KMainWindow( 0, "logik" ),
      m_view(new logikView(this)),
      m_printer(0)
{
    setAcceptDrops(true);
    setCentralWidget(m_view);
    setupActions();
    statusBar()->show();

    setupGUI();

    connect(m_view, SIGNAL(signalChangeStatusbar(const QString&)),
            this,   SLOT(changeStatusbar(const QString&)));
    connect(m_view, SIGNAL(signalChangeCaption(const QString&)),
            this,   SLOT(changeCaption(const QString&)));
    connect(m_view, SIGNAL(signalChangeState(const QString&)),this,SLOT(slotStateChanged(const QString&)));
    stateChanged("not_loaded");
}

logik::~logik()
{
    if (m_view->isChanged())
     {
       int result = KMessageBox::warningYesNo(this,i18n("Document has been modified. Do you want to save your changes or discard them?"),i18n("Quit"),i18n("Save"),i18n("Discard"));
       if (result==KMessageBox::Yes)
        fileSave();
     }
}

void logik::load(const KURL& url)
{
    if (m_view->isChanged())
     {
       int result = KMessageBox::warningYesNoCancel(this,i18n("Document has been modified. Do you want to save your changes or discard them?"),i18n("Load Document"),i18n("Save"),i18n("Discard"));
       if (result==KMessageBox::Yes)
        fileSave();
       else if (result==KMessageBox::Cancel)
        return;
     }
    setCaption(url.prettyURL());
    m_view->openURL(url);
}

void logik::setupActions()
{
    KStdAction::openNew(this, SLOT(fileNew()), actionCollection());
    KStdAction::open(this, SLOT(fileOpen()), actionCollection());
    KStdAction::save(this, SLOT(fileSave()), actionCollection());
    KStdAction::saveAs(this, SLOT(fileSaveAs()), actionCollection());
    KStdAction::print(this, SLOT(filePrint()), actionCollection());
    KStdAction::quit(kapp, SLOT(quit()), actionCollection());

    //KStdAction::preferences(this, SLOT(optionsPreferences()), actionCollection());

    // this doesn't do anything useful.  it's just here to illustrate
    // how to insert a custom menu and menu item
    (void) new KAction(i18n("&Show"), 0,
                                  this, SLOT(openSolution()),
                                  actionCollection(), "solution_show");
    (void) new KAction(i18n("Insert &Row"), 0,
                                  this, SLOT(insertRow()),
                                  actionCollection(), "insert_row");
    (void) new KAction(i18n("Insert &Column"), 0,
                                  this, SLOT(insertColumn()),
                                  actionCollection(), "insert_column");
}

void logik::saveProperties(KConfig *config)
{
    // the 'config' object points to the session managed
    // config file.  anything you write here will be available
    // later when this app is restored

    if (!m_view->currentURL().isEmpty()) {
#if KDE_IS_VERSION(3,1,3)
        config->writePathEntry("lastURL", m_view->currentURL());
#else
        config->writeEntry("lastURL", m_view->currentURL());
#endif
    }
}

void logik::readProperties(KConfig *config)
{
    // the 'config' object points to the session managed
    // config file.  this function is automatically called whenever
    // the app is being restored.  read in here whatever you wrote
    // in 'saveProperties'

    QString url = config->readPathEntry("lastURL");

    if (!url.isEmpty())
        m_view->openURL(KURL(url));
}

void logik::dragEnterEvent(QDragEnterEvent *event)
{
    // accept uri drops only
    event->accept(KURLDrag::canDecode(event));
}

void logik::dropEvent(QDropEvent *event)
{
    // this is a very simplistic implementation of a drop event.  we
    // will only accept a dropped URL.  the Qt dnd code can do *much*
    // much more, so please read the docs there
    KURL::List urls;

    // see if we can decode a URI.. if not, just ignore it
    if (KURLDrag::decode(event, urls) && !urls.isEmpty())
    {
        // okay, we have a URI.. process it
        const KURL &url = urls.first();

        // load in the file
        load(url);
    }
}

void logik::fileNew()
{
    if (m_view->isChanged())
     {
       int result = KMessageBox::warningYesNoCancel(this,i18n("Document has been modified. Do you want to save your changes or discard them?"),i18n("New Document"),i18n("Save"),i18n("Discard"));
       if (result==KMessageBox::Yes)
        fileSave();
       else if (result==KMessageBox::Cancel)
        return;
     }
    m_view->Create();
}

void logik::fileOpen()
{
    if (m_view->isChanged())
     {
       int result = KMessageBox::warningYesNoCancel(this,i18n("Document has been modified. Do you want to save your changes or discard them?"),i18n("New Document"),i18n("Save"),i18n("Discard"));
       if (result==KMessageBox::Yes)
        fileSave();
       else if (result==KMessageBox::Cancel)
        return;
      m_view->setChanged(false); 
     }
    KURL url = KFileDialog::getOpenURL(QString::null, QString::null, this, i18n("Open Location"));
    if (!url.isEmpty())
        load(url);
}

void logik::fileSave()
{
    if(!m_view->currentURL())
     fileSaveAs();
    m_view->saveURL(m_view->currentURL());
}

void logik::fileSaveAs()
{
    KURL file_url = KFileDialog::getSaveURL();
    if (!file_url.isEmpty() && file_url.isValid())
    {
        m_view->saveURL( file_url);
    }
}

void logik::filePrint()
{
    
      if (!m_printer) m_printer = new KPrinter;
      if (m_printer->setup(this))
      {
       QCanvas *canvas = m_view->canvas();
       QPainter p;
       p.begin(m_printer);
       QPaintDeviceMetrics metrics(m_printer);
       canvas->drawArea(canvas->rect(),&p);
       p.end();
      }
}

void logik::optionsPreferences()
{
    // popup some sort of preference dialog, here
    //m_view->openSolution();
    /*logikPreferences dlg;
    if (dlg.exec())
    {
        // redo your settings
    }*/
}

void logik::changeStatusbar(const QString& text)
{
    statusBar()->message(text);
}

void logik::changeCaption(const QString& text)
{
    setCaption(text);
}

void logik::slotOpen(const QString& string)
{
    m_view->openURL(string);
}
#include "logik.moc"


/*!
    \fn logik::openSolution()
 */
void logik::openSolution()
{
    m_view->openSolution();
}


/*!
    \fn logik::insertColumn()
 */
void logik::insertColumn()
{
    m_view->getView()->setInsertMode( 2 );
}


/*!
    \fn logik::insertRow()
 */
void logik::insertRow()
{
    m_view->getView()->setInsertMode( 1 );
}
