/***************************************************************************
 *   Copyright (C) 2006 by Stefan Hirche                                   *
 *   hirche@astro.univie.ac.at                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "logikview.h"
#include "createform.h"
#include "solutionform.h"


#include <qpainter.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qcanvas.h>
#include <qlayout.h>
#include <qdom.h>

#include "myview.h"
#include <kurl.h>


#include <kmessagebox.h>
#include <krun.h>
#include <klocale.h>

logikView::logikView(QWidget *parent)
    : QWidget(parent),
      DCOPObject("logikIface")
{
    QHBoxLayout *top_layout = new QHBoxLayout(this);
    top_layout->setAutoAdd(true);
    top_layout->setResizeMode(QLayout::Minimum);
    view = new myview(this);
    connect (view,SIGNAL(signalChanged()),this,SLOT(slotChanged()));
    setChanged(false);
}

logikView::~logikView()
{
}


QString logikView::currentURL()
{
    return URL.url();
}

void logikView::openURL(QString url)
{
    openURL(KURL(url));
}

void logikView::openURL(const KURL& url)
{
    QFile file(url.path());
    if (domParser(&file)==0)
     {
      view->setGrid(view->gridWidth(),view->gridHeight());
      view->setStateArray(gridstatearray);
      view->paintNumbers(numbersarray);
      view->setNumbersArray(numbersstatearray);
      view->setSolutionArray(solutionarray);
      view->setChanged(false);
      view->setEnabled(true);
      URL=url;
      emit signalChangeState("default");
     }
    file.close();
}

/*!
    \fn logikView::saveURL(const KURL&url)
 */
void logikView::saveURL(const KURL&url)
{
    QFile file(url.path());
    
    file.open(IO_WriteOnly);
    QTextStream stream(&file);
    
    URL=url;
    
    QString string_width = QString("%1").arg(view->gridWidth());
    QString string_height = QString("%1").arg(view->gridHeight());
    QString string_colors = QString("%1").arg(2);
    QString string_image = QString("Image");
    QString string_numbers;
    QString string_numbersdata;
    QString string_griddata;
    QString string_data;
    
    for (int i=0;i<view->gridHeight()*view->gridWidth();i++)
     string_data.append(QString("%1").arg(solutionarray[i]));
    for (int i=0;i<numbersstatearray.size();i++)
     string_numbersdata.append(QString("%1").arg(numbersstatearray[i]));
    for (int i=0;i<view->gridHeight()*view->gridWidth();i++)
     string_griddata.append(QString("%1").arg(gridstatearray[i]));
    for (int i=0;i<numbersarray.size();i++)
     {
      string_numbers.append(QString("%1x").arg(numbersarray[i]));
     }
    QDomDocument doc;
    QDomElement el_logik = doc.createElement("logik");
    QDomElement el_general = doc.createElement("general");
    QDomElement el_width = doc.createElement("width");
    QDomElement el_height = doc.createElement("height");
    QDomElement el_colors = doc.createElement("colors");
    QDomElement el_solution = doc.createElement("solution");
    QDomElement el_image = doc.createElement("image");
    QDomElement el_data = doc.createElement("data");
    QDomElement el_numbers = doc.createElement("numbers");
    QDomElement el_state = doc.createElement("state");
    QDomElement el_griddata = doc.createElement("griddata");
    QDomElement el_numbersdata = doc.createElement("numbersdata");
    
    QDomText text_width = doc.createTextNode(string_width);
    QDomText text_height = doc.createTextNode(string_height);
    QDomText text_colors = doc.createTextNode(string_colors);
    QDomText text_image = doc.createTextNode(string_image);
    QDomText text_data = doc.createTextNode(string_data);
    QDomText text_numbers = doc.createTextNode(string_numbers);
    QDomText text_griddata = doc.createTextNode(string_griddata);
    QDomText text_numbersdata = doc.createTextNode(string_numbersdata);
    
    
    
    doc.appendChild(el_logik);
    el_logik.appendChild(el_general);
    el_general.appendChild(el_width);
    el_general.appendChild(el_height);
    el_general.appendChild(el_colors);
    el_logik.appendChild(el_solution);
    el_solution.appendChild(el_image);
    el_solution.appendChild(el_data);
    el_solution.appendChild(el_numbers);
    el_logik.appendChild(el_state);
    el_state.appendChild(el_griddata);
    el_state.appendChild(el_numbersdata);
    
    el_width.appendChild(text_width);
    el_height.appendChild(text_height);
    el_colors.appendChild(text_colors);
    el_image.appendChild(text_image);
    el_data.appendChild(text_data);
    el_numbers.appendChild(text_numbers);
    el_griddata.appendChild(text_griddata);
    el_numbersdata.appendChild(text_numbersdata);
    
    QDomNode xmlNode = doc.createProcessingInstruction("xml",
                          "version=\"1.0\" encoding=\"ISO-8859-1\"");
    doc.insertBefore(xmlNode, doc.firstChild());
    
    doc.save(stream,4);
    file.close();
    
    view->setChanged(false);
    setChanged(false);
    emit signalChangeCaption(URL.path());
    emit signalChangeState("default");
}



void logikView::slotSetTitle(const QString& title)
{
    emit signalChangeCaption(title);
}



/*!
    \fn logikView::changed()
 */
void logikView::slotChanged()
{
    QString string(URL.path());
    string.append(" [").append(i18n("modified")).append("]");
    emit signalChangeCaption(string);
    emit signalChangeState("modified");
    setChanged(true);
}

/*!
    \fn logikView::isChanged()
 */
bool logikView::isChanged()
{
    return changed;
}

/*!
    \fn logikView::setChanged(bool state)
 */
void logikView::setChanged(bool state)
{
    changed = state;
}


/*!
    \fn logikView::canvas()
 */
QCanvas* logikView::canvas()
{
    return view->canvas();
}

/*!
    \fn logikView::canvasOpened()
 */
bool logikView::canvasOpened()
{
    return view->canvasOpened();
}

#include "logikview.moc"


/*!
    \fn logikView::domParser(QIODevice *device)
 */
int logikView::domParser(QIODevice *device)
{
    QDomDocument doc;
    QString errorStr;
    int errorLine, errorColumn;
    
    if (!doc.setContent(device,true))
     {
      KMessageBox::error(this,QString(i18n("File cannot be loaded. Error %1 in line %2, column %3")).arg(errorStr).arg(errorLine).arg(errorColumn));
      return 1;
     }
    QDomElement root = doc.documentElement();
    if (root.tagName()!="logik")
     {
      KMessageBox::error(this,i18n("The file is not a logiK file"));
      return 1;
     }
    QDomNode node = root.firstChild();
    while(!node.isNull())
     {
      if (node.toElement().tagName() == "solution")
       parseElement(node.toElement(),"solution");
      if (node.toElement().tagName() == "state")
       parseElement(node.toElement(),"state");
      if (node.toElement().tagName() == "general")
       parseElement(node.toElement(),"general");
      node = node.nextSibling();
      }
     return 0;
}

/*!
    \fn logikView::parseElement(const QDomElement &element, const QString& string)
 */
void logikView::parseElement(const QDomElement &element, const QString& string)
{
    QDomNode node = element.firstChild();
    int k;
    while(!node.isNull())
     {
      QString content = node.firstChild().toText().data();
      if (node.toElement().tagName() == "width")
       view->setGridWidth(content.toInt());
      if (node.toElement().tagName() == "height")
       view->setGridHeight(content.toInt());
      if (node.toElement().tagName() == "colors")
       view->setColors(content.toInt());
      if (node.toElement().tagName() == "data")
       {
        solutionarray.resize(content.length());
        for (uint i=0;i<content.length();i++)
         solutionarray[i]=content[i].digitValue();
       }
      if (node.toElement().tagName() == "numbers")
       {
        numbersarray.resize(content.length());
        k = 0;
        for (uint i=0;i<content.length();i++)
         {
          numbersarray[k]=0;
          for (;content[i].isDigit() && i<content.length();i++)
           {
            numbersarray[k]=numbersarray[k]*10+content[i].digitValue();
           }
          k++;
         }
        numbersarray.resize(k);
       }
      if (node.toElement().tagName() == "numbersdata")
       {
        numbersstatearray.resize(content.length());
        for (uint i=0;i<content.length();i++)
         numbersstatearray[i]=content[i].digitValue();
       }
      if (node.toElement().tagName() == "griddata")
       {
        gridstatearray.resize(content.length());
        for (uint i=0;i<content.length();i++)
         gridstatearray[i]=content[i].digitValue();
       }
      node = node.nextSibling();
     }
}


/*!
    \fn logikView::Create();
 */
void logikView::Create()
{
    createForm *createDlg = new createForm(this);
    createDlg->show();
    connect (createDlg,SIGNAL(signalCreateAction(QMemArray<int>,int,int)),this,SLOT(slotCreated(QMemArray<int>,int,int)));
}


/*!
    \fn logikView::openSolution()
 */
void logikView::openSolution()
{
   solutionForm *form = new solutionForm(this);
   form->show();
   myview *solView = form->solutionView();
   solView->setGrid(view->gridWidth(),view->gridHeight());
   solView->setStateArray(solutionarray);
}


/*!
    \fn logikView::slotCreated(QMemArray<int> array, int x, int y)
 */
void logikView::slotCreated(QMemArray<int> array, int x, int y)
{
   solutionarray=array;
   gridstatearray.resize(x*y);
   for (int i=0;i<x*y;i++)
    gridstatearray[i]=0;
   view->setGridWidth(x);
   view->setGridHeight(y);
   solutionToNumbers();
   view->setGrid(x,y);
   view->setStateArray(gridstatearray);
   view->paintNumbers(numbersarray);
   view->setNumbersArray(numbersstatearray);
   view->setSolutionArray(solutionarray);
   URL=KURL();
   emit signalChangeCaption("");
   slotChanged();
   view->setEnabled(true);
   emit signalChangeState("default");
   emit signalChangeState("modified");
}


/*!
    \fn logikView::solutionToNumbers()
 */
void logikView::solutionToNumbers()
{
    qDebug("Start solutionToNumbers");
    int l=0,k,j,i;
    numbersarray.resize(view->gridWidth()*view->gridHeight());
    qDebug("Start first loop");
    for (i=view->gridWidth(); i>0; i--)
     {
      for (j=view->gridHeight(); j>0; j--)
       {
        for (k=0;solutionarray[(i-1)*view->gridHeight()+j-1]==1 && j>0; j--)
         {
          k++;
         }
        if (k>0)
         numbersarray[l++]=k;
       }
      numbersarray[l++]=0;
     }
   qDebug("End first loop");
     
    qDebug("Start second loop");
    for (j=view->gridHeight(); j>0; j--)
     {
      for (i=view->gridWidth(); i>0; i--)
       {
        for (k=0;solutionarray[(i-1)*view->gridHeight()+j-1]==1 && i>0; i--)
         {
          //qDebug(QString("i: %1 j:%2 test: %3").arg(i).arg(j).arg(view->gridHeight()));
          k++;
         }
        if (k>0)
         numbersarray[l++]=k;
       }
      numbersarray[l++]=0;
     }
    qDebug("End second loop");
    qDebug(QString("size %1, l %2").arg(numbersarray.size()).arg(l));
    numbersarray.resize(l);
    numbersstatearray.resize(l);
     for (int i=0;i<l;i++)
      numbersstatearray[i]=0;
      qDebug("End solutionToNumbers");
}


/*!
    \fn logikView::getView()
 */
myview* logikView::getView()
{
    return view;
}
