/***************************************************************************
 *   Copyright (C) 2006 by Stefan Hirche   *
 *   hirche@astro.univie.ac.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef _KLOGICVIEW_H_
#define _KLOGICVIEW_H_

#include <qwidget.h>
#include <logikiface.h>
#include <kurl.h>
#include <qcanvas.h>
#include <qdom.h>
#include <qiodevice.h>

class QPainter;
class KURL;
class myview;


/**
 * This is the main view class for logik.  Most of the non-menu,
 * non-toolbar, and non-statusbar (e.g., non frame) GUI code should go
 * here.
 *
 * This logik uses an HTML component as an example.
 *
 * @short Main view
 * @author Stefan Hirche <hirche@astro.univie.ac.at>
 * @version 0.1
 */
class logikView : public QWidget, public logikIface
{
    Q_OBJECT
public:

    logikView(QWidget *parent);

    virtual ~logikView();

    QString currentURL();
    virtual void openURL(QString url);
    virtual void openURL(const KURL& url);

    void saveURL(const KURL& url);
    
    bool isChanged();
    void setChanged(bool state);
    
    QCanvas* canvas();
    bool canvasOpened();
    int domParser(QIODevice *device);
    void parseElement(const QDomElement& element, const QString& string);
    void Create();;
    void openSolution();
    myview* getView();

signals:
    void signalChangeStatusbar(const QString& text);
    void signalChangeCaption(const QString& text);
    void signalChangeState(const QString& text);
    
private:
    myview *view;
    KURL URL;
    bool changed;
    QMemArray<int> gridstatearray;
    QMemArray<int> numbersstatearray;
    QMemArray<int> numbersarray;
    QMemArray<int> solutionarray;

private slots:
    void slotSetTitle(const QString& title);

public slots:
    void slotChanged();
    void slotCreated(QMemArray<int> array, int x, int y);
private:
    void solutionToNumbers();
};

#endif // _KLOGICVIEW_H_
