/***************************************************************************
 *   Copyright (C) 2006 by Stefan Hirche   *
 *   hirche@astro.univie.ac.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "myview.h"
#include "myrectangle.h"
#include "mynumbers.h"
#include <kmessagebox.h>
#include <qpainter.h>
#include <qcursor.h>

myview::myview(QWidget *parent)
 : QCanvasView(parent)
{
	
	setYShift(150);
	setXShift(150);
	setCellHeight(12);
	setCellWidth(12);
	setEnabled(false);
	setChanged(false);
	openCanvas=false;
	setXAdd(50);
	setYAdd(50);
	setInsertMode(0);
	
}

myview::~myview()
{
}


/*!
    \fn myview::contentsMousePressEvent(QMouseEvent* e)
 */
void myview::contentsMousePressEvent(QMouseEvent* e)
{
   QValueList<myrectangle*>::iterator it;
   QValueList<mynumbers*>::iterator nt;
   int status,i;
   
   if (insertmode == 0)
   {
    if (e->x()>=xshift && e->y()>=yshift)
     {
      if (e->state()==Qt::ShiftButton)
          status=0;
      else if (e->button()==Qt::LeftButton)
          status=1;
      else if(e->button()==Qt::RightButton)
          status=2;
      else
          status=0;
      for (it = rectlist.begin(),i=0; it != rectlist.end(); it++,i++ )
        if((*it)->x()<e->x() && (*it)->x()>e->x()-cellwidth-1 && (*it)->y()<e->y() &&  (*it)->y()>e->y()-cellheight-1)
        {
          if ((*it)->state()!=status)
           {
            (*it)->setState(status);
            gridstatearray[i]=status;
            if (!isChanged())
             {
              setChanged(true);
              emit signalChanged();
             }
           }
        }
     }
     else
      {
       if (e->button()==Qt::LeftButton)
        status=1;
       else
        status=0;
       for (nt = numlist.begin(),i=0; nt != numlist.end(); nt++,i++)
        if((*nt)->startPoint().x()>e->x() && (*nt)->endPoint().x()<e->x() && (*nt)->endPoint().y()>e->y() && (*nt)->startPoint().y()<e->y())
         {
          if ((*nt)->state()!=status)
          {
           numberstatearray[i]=status;
           (*nt)->setState(status);
           if (!isChanged())
            {
             setChanged(true);
             emit signalChanged();
            }
          }
         }
      }
     }
    else if (insertmode == 1) {
      setInsertMode(0);
      if (e->y()>=yshift) {
      for (it = rectlist.begin(),i=0; it != rectlist.end(); it++,i++ )
        if((*it)->y()<e->y() &&  (*it)->y()>e->y()-cellheight-1)
        {
          status = solutionarray[i]==1?1:2;
          if ((*it)->state()!=status)
           {
            (*it)->setState(status);
            gridstatearray[i]=status;
            if (!isChanged())
             {
              setChanged(true);
              emit signalChanged();
             }
           }
        }
      for (nt = numlist.begin(),i=0; nt != numlist.end(); nt++,i++)
        if((*nt)->endPoint().y()>e->y() && (*nt)->startPoint().y()<e->y())
         {
          status = 1;
          if ((*nt)->state()!=status)
          {
           numberstatearray[i]=status;
           (*nt)->setState(status);
           if (!isChanged())
            {
             setChanged(true);
             emit signalChanged();
            }
          }
         }
       }
      }
    else if (insertmode == 2) {
      setInsertMode(0);
      if (e->x()>=xshift) {
      for (it = rectlist.begin(),i=0; it != rectlist.end(); it++,i++ )
        if((*it)->x()<e->x() && (*it)->x()>e->x()-cellwidth-1)
        {
          status = solutionarray[i]==1?1:2;
          if ((*it)->state()!=status)
           {
            (*it)->setState(status);
            gridstatearray[i]=status;
            if (!isChanged())
             {
              setChanged(true);
              emit signalChanged();
             }
           }
        }
      for (nt = numlist.begin(),i=0; nt != numlist.end(); nt++,i++)
        if((*nt)->startPoint().x()>e->x() && (*nt)->endPoint().x()<e->x())
         {
          if ((*nt)->state()!=status)
          {
           numberstatearray[i]=status;
           (*nt)->setState(status);
           if (!isChanged())
            {
             setChanged(true);
             emit signalChanged();
            }
          }
         }
       }
      }
    
    
   c->update();
}


/*!
    \fn myview::paintGrid(int width, int height)
 */
void myview::paintGrid(int width, int height)
{
    int i,j;
    rectlist.clear();
    c->resize(width*cellwidth+xshift+xadd,height*cellheight+yshift+yadd);
    setMinimumWidth(c->width()+1);
    setMinimumHeight(c->height()+1);
    for (i=0;i<width;i++)
      for (j=0;j<height;j++)
    	paintGridCell(i*cellwidth+xshift,j*cellheight+yshift);
    for (i=0;i<=width;i++)
      paintGridLine(i*cellwidth+xshift,5,i*cellwidth+xshift,height*cellheight+yshift,(i%5)?1:2);
    for (i=0;i<=height;i++)
      paintGridLine(5,i*cellheight+yshift,width*cellwidth+xshift,i*cellheight+yshift,(i%5)?1:2);
    c->update();
}

/*!
    \fn myview::paintGridCell(int x, int y)
 */
void myview::paintGridCell(int x, int y)
{
    myrectangle *rect = new myrectangle(c);
    rect->setZ(10);
    rect->setX(x);
    rect->setY(y);
    rect->setPen(QPen(white,0));
    rect->setSize(cellwidth+1,cellheight+1);
    rect->setState(0);
    rect->show();
    rectlist.append(rect);
}

/*!
    \fn myview::paintGridLine(int x1, int y1, int x2, int y2, int thick)
 */
void myview::paintGridLine(int x1, int y1, int x2, int y2, int thick)
{
    QCanvasLine *line = new QCanvasLine(c);
    line->setPoints(x1,y1,x2,y2);
    line->setPen(QPen(black,thick));
    line->setZ(20);
    line->show();
}


/*!
    \fn myview::gridStateArray()
 */
QMemArray<int> myview::gridStateArray()
{
  return gridstatearray;
}

/*!
    \fn myview::setStateArray(QMemArray<int> gridarray)
 */
void myview::setStateArray(QMemArray<int> gridarray)
{
    int i;
    gridstatearray=gridarray;
    QValueList<myrectangle*>::iterator rt;
    for(rt=rectlist.begin(),i=0;rt!=rectlist.end();rt++,i++)
     {
      (*rt)->setState(gridstatearray[i]);
     }
    c->update();
}

/*!
    \fn myview::setNumbersArray(QMemArray<int> numarray)
 */
 void myview::setNumbersArray(QMemArray<int> numarray)
{
 int i;
 numberstatearray = numarray;
 QValueList<mynumbers*>::iterator rt;
    for(rt=numlist.begin(),i=0;rt!=numlist.end();rt++,i++)
     {
      (*rt)->setState(numberstatearray[i]);
     }
    c->update();

}

/*!
    \fn myview::isChanged()
 */
bool myview::isChanged()
{
    return changed;
}

/*!
    \fn myview::setChanged(bool state)
 */
void myview::setChanged(bool state)
{
    changed=state;
}


/*!
    \fn myview::canvas()
 */
QCanvas* myview::canvas()
{
    return c;
}

/*!
    \fn myview::canvasOpened()
 */
bool myview::canvasOpened()
{
    return openCanvas;
}


/*!
    \fn myview::setYShift(int shift)
 */
void myview::setYShift(int shift)
{
    yshift=shift;
}

/*!
    \fn myview::setXShift(int shift)
 */
void myview::setXShift(int shift)
{
    xshift=shift;
}

/*!
    \fn myview::xShift()
 */
int myview::xShift()
{
    return xshift;
}

/*!
    \fn myview::yShift()
 */
int myview::yShift()
{
    return yshift;
}


/*!
    \fn myview::setCellWidth(int width)
 */
void myview::setCellWidth(int width)
{
    cellwidth=width;
}

/*!
    \fn myview::setCellHeight(int height)
 */
void myview::setCellHeight(int height)
{
    cellheight=height;
}

/*!
    \fn myview::cellWidth()
 */
int myview::cellWidth()
{
    return cellwidth;
}

/*!
    \fn myview::cellHeight()
 */
int myview::cellHeight()
{
    return cellheight;
}


/*!
    \fn myview::gridWidth()
 */
int myview::gridWidth()
{
    return gridwidth;
}

/*!
    \fn myview::gridHeight()
 */
int myview::gridHeight()
{
    return gridheight;
}

/*!
    \fn myview::setGridHeight(int value)
 */
void myview::setGridHeight(int value)
{
    gridheight=value;
}

/*!
    \fn myview::setGridWidth(int value)
 */
void myview::setGridWidth(int value)
{
    gridwidth=value;
}



/*!
    \fn myview::setGrid(int width, int height)
 */
void myview::setGrid(int width, int height)
{
    c = new QCanvas();
    openCanvas=true;
    setCanvas(c);
    
    setGridHeight(height);
    setGridWidth(width);
    paintGrid(gridwidth,gridheight);
    
    c->update();
}


/*!
    \fn myview::colors();
 */
int myview::colors()
{
    return color;
}


/*!
    \fn myview::setColors(int number)
 */
void myview::setColors(int number)
{
    color = number;
}


/*!
    \fn myview::xAdd()
 */
int myview::xAdd()
{
    return xadd;
}


/*!
    \fn myview::yAdd()
 */
int myview::yAdd()
{
    return yadd;
}


/*!
    \fn myview::setYAdd(int y)
 */
void myview::setYAdd(int y)
{
    yadd = y;
}


/*!
    \fn myview::setXAdd(int x)
 */
void myview::setXAdd(int x)
{
    xadd = x;
}


/*!
    \fn myview::paintNumber(int x, int y, int num, int orientation)
 */
void myview::paintNumber(int x, int y, int num, int orientation)
{
    QCanvasText *text = new QCanvasText(QString("%1").arg(num),c);
    mynumbers *line = new mynumbers(c);
    line->setPoints(x+cellwidth,y,x,y+cellheight);
    line->setState(0);
    text->setTextFlags(Qt::AlignCenter);
    text->setColor(black);
    text->setX(x+cellwidth/2);
    text->setY(y+cellheight/2);
    text->setZ(15);
    text->setFont(QFont("Helvetica",cellwidth*3/5));
    text->show();
    line->show();
    numlist.append(line);
}


/*!
    \fn myview::paintNumbers(QMemArray<int> array)
 */
void myview::paintNumbers(QMemArray<int> array)
{
    int x=1,y=1;
    uint i=0;
    numlist.clear();
    for(;i<array.size() && x<=gridwidth;i++)
     {
      if (array[i]==0)
       {
        y=1;
        x++;
       }
      else
       {
        paintNumber(xshift+gridwidth*cellwidth-x*cellwidth,-2+yshift-y*cellheight,array[i],0);
        y++;
       }
     }
    x=1,y=1;
    for(;i<array.size();i++)
     {
      if (array[i]==0)
       {
        x=1;
        y++;
       }
      else
       {
        paintNumber(xshift-x*cellwidth-2,yshift+gridheight*cellheight-y*cellheight,array[i],1);
        x++;
       }
     }
     
     
}


/*!
    \fn myview::numStateArray()
 */
QMemArray<int> myview::numStateArray()
{
    return numberstatearray;
}


/*!
    \fn myview::insertMode()
 */
int myview::insertMode()
{
    return insertmode;
}


/*!
    \fn myview::setInsertMode(int mode)
 */
void myview::setInsertMode(int mode)
{
    insertmode = mode;
    if (insertmode == 0) 
     setCursor(QCursor(Qt::ArrowCursor));
    else setCursor(QCursor(Qt::PointingHandCursor));
}


/*!
    \fn myview::setSolutionArray(QMemArray<int> solarray)
 */
void myview::setSolutionArray(QMemArray<int> solarray)
{
    solutionarray = solarray;
}
