/***************************************************************************
 *   Copyright (C) 2006 by Stefan Hirche   *
 *   hirche@astro.univie.ac.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MYVIEW_H
#define MYVIEW_H

#include <qcanvas.h>
#include "myrectangle.h"
#include "mynumbers.h"


/**
	@author Stefan Hirche <hirche@astro.univie.ac.at>
*/
class myview : public QCanvasView
{
Q_OBJECT
public:
    myview(QWidget *parent);
    ~myview();
    
    void paintGrid(int width, int height);
    void paintGridCell(int x, int y);
    void paintGridLine(int x1, int y1, int x2, int y2, int thick);
    
    int cellHeight();
    int cellWidth();
    void setCellWidth(int width);
    void setCellHeight(int height);
    
    int yShift();
    int xShift();
    void setYShift(int shift);
    void setXShift(int shift);
    
    int gridWidth();
    int gridHeight();
    void setGridHeight(int value);
    void setGridWidth(int value);
    
    QMemArray<int> gridStateArray();
    void setStateArray(QMemArray<int> gridarray);
    void setNumbersArray(QMemArray<int> numarray);
    bool isChanged();
    void setChanged(bool state);
    
    QCanvas* canvas();
    bool canvasOpened();
    void setGrid(int width, int height);
    int colors();;
    void setColors(int number);
    int xAdd();
    int yAdd();
    void setYAdd(int y);
    void setXAdd(int x);
    void paintNumber(int x, int y, int num, int orientation);
    void paintNumbers(QMemArray<int> array);
    QMemArray<int> numStateArray();
    int insertMode();
    void setInsertMode(int mode);
    void setSolutionArray(QMemArray<int> solarray);

private:
    void contentsMousePressEvent(QMouseEvent* e);
private:
    QCanvas *c;
    QValueList<myrectangle*> rectlist;
    QValueList<mynumbers*> numlist;
    
    QMemArray<int> gridstatearray;
    QMemArray<int> numberstatearray;
    QMemArray<int> solutionarray;
    
    int cellwidth;
    int cellheight;
    int yshift;
    int xshift;
    int xadd;
    int yadd;
    int gridwidth;
    int gridheight;
    int color;
    
    bool changed;
    bool openCanvas;
    int insertmode;
signals:
    void signalChanged();
};

#endif
